---------------------------------------------------------------------------------------------
--ATF version: 2.2
--Created date: 12/Nov/2015
--Author: Ta Thanh Dong
---------------------------------------------------------------------------------------------
Test = require('connecttest')
require('cardinalities')
local events = require('events')
local mobile_session = require('mobile_session')
local config = require('config')


---------------------------------------------------------------------------------------------
-----------------------------Required Shared Libraries---------------------------------------
---------------------------------------------------------------------------------------------
local commonFunctions = require('user_modules/shared_testcases/commonFunctions')
local commonSteps = require('user_modules/shared_testcases/commonSteps')
local commonTestCases = require('user_modules/shared_testcases/commonTestCases')
local integerParameterInNotification = require('user_modules/shared_testcases/testCasesForIntegerParameterInNotification')
local floatParameterInNotification = require('user_modules/shared_testcases/testCasesForFloatParameterInNotification')
local stringParameterInNotification = require('user_modules/shared_testcases/testCasesForStringParameterInNotification')
local enumParameterInNotification = require('user_modules/shared_testcases/testCasesForEnumerationParameterInNotification')
local booleanParameterInNotification = require('user_modules/shared_testcases/testCasesForBooleanParameterInNotification')
local testCasesForPolicyTable = require('user_modules/shared_testcases/testCasesForPolicyTable')

require('user_modules/AppTypes')

APIName = "OnVehicleData" -- set API name
Debug = {} 
Apps = {}
Apps[1] = {}
Apps[1].storagePath = config.SDLStoragePath..config.application1.registerAppInterfaceParams.appID.. "_" .. config.deviceMAC.. "/"
Apps[1].appName = config.application1.registerAppInterfaceParams.appName 
Apps[1].isMedia = commonFunctions:isMediaApp()
--Set audioStreamingState
if Apps[1].isMedia then
	Apps[1].audioStreamingState = "AUDIBLE"
else
	Apps[1].audioStreamingState = "NOT_AUDIBLE"
end
	
Apps[2] = {}
Apps[2].appName = config.application2.registerAppInterfaceParams.appName 	

---------------------------------------------------------------------------------------------
------------------------------------------Common functions-----------------------------------
---------------------------------------------------------------------------------------------



local SVDValues = {		gps						= "VEHICLEDATA_GPS", 
						speed					= "VEHICLEDATA_SPEED", 
						rpm						= "VEHICLEDATA_RPM", 
						fuelLevel				= "VEHICLEDATA_FUELLEVEL", 
						fuelLevel_State			= "VEHICLEDATA_FUELLEVEL_STATE", 
						instantFuelConsumption	= "VEHICLEDATA_FUELCONSUMPTION", 
						externalTemperature		= "VEHICLEDATA_EXTERNTEMP", 
						prndl					= "VEHICLEDATA_PRNDL", 
						tirePressure			= "VEHICLEDATA_TIREPRESSURE", 
						odometer				= "VEHICLEDATA_ODOMETER", 
						beltStatus				= "VEHICLEDATA_BELTSTATUS", 
						bodyInformation			= "VEHICLEDATA_BODYINFO", 
						deviceStatus			= "VEHICLEDATA_DEVICESTATUS", 
						driverBraking			= "VEHICLEDATA_BRAKING", 
						wiperStatus				= "VEHICLEDATA_WIPERSTATUS", 
						headLampStatus			= "VEHICLEDATA_HEADLAMPSTATUS", 
						engineTorque			= "VEHICLEDATA_ENGINETORQUE", 
						accPedalPosition		= "VEHICLEDATA_ACCPEDAL", 
						steeringWheelAngle		= "VEHICLEDATA_STEERINGWHEEL", 
						eCallInfo				= "VEHICLEDATA_ECALLINFO", 
						airbagStatus			= "VEHICLEDATA_AIRBAGSTATUS", 
						emergencyEvent			= "VEHICLEDATA_EMERGENCYEVENT", 
						clusterModeStatus		= "VEHICLEDATA_CLUSTERMODESTATUS", 
						myKey					= "VEHICLEDATA_MYKEY"}
						

local allVehicleData = {	"gps", 
							"speed", 
							"rpm", 
							"fuelLevel", 
							"fuelLevel_State", 
							"instantFuelConsumption", 
							"externalTemperature", 
							"prndl", 
							"tirePressure", 
							"odometer", 
							"beltStatus", 
							"bodyInformation", 
							"deviceStatus", 
							"driverBraking", 
							"wiperStatus", 
							"headLampStatus", 
							"engineTorque", 
							"accPedalPosition", 
							"steeringWheelAngle", 
							"eCallInfo", 
							"airbagStatus", 
							"emergencyEvent", 
							"clusterModeStatus", 
							"myKey"}
							
							
local vehicleData = {"gps"}
local infoMessageValue = string.rep("a",1000)

function DelayedExp(time)
	local event = events.Event()
  event.matches = function(self, e) return self == e end
  EXPECT_EVENT(event, "Delayed event")
  :Timeout(time + 1000)
  RUN_AFTER(function()
              RAISE_EVENT(event, event)
            end, time)
end



function Test:subscribeVehicleDataSuccess(paramsSend, infoMessage)
	
	local function setSVDRequest(paramsSend)
		local temp = {}	
		for i = 1, #paramsSend do		
			temp[paramsSend[i]] = true
		end	
		return temp
	end
	
	local function setSVDResponse(paramsSend, vehicleDataResultCode)
		local temp = {}
		local vehicleDataResultCodeValue = ""
		
		if vehicleDataResultCode ~= nil then
			vehicleDataResultCodeValue = vehicleDataResultCode
		else
			vehicleDataResultCodeValue = "SUCCESS"
		end
		
		for i = 1, #paramsSend do
			if  paramsSend[i] == "clusterModeStatus" then
				temp["clusterModes"] = {					
							resultCode = vehicleDataResultCodeValue, 
							dataType = SVDValues[paramsSend[i]]
					}
			else
				temp[paramsSend[i]] = {					
							resultCode = vehicleDataResultCodeValue, 
							dataType = SVDValues[paramsSend[i]]
					}
			end
		end	
		return temp
	end
	
	local function createSuccessExpectedResult(response, infoMessage)
		response["success"] = true
		response["resultCode"] = "SUCCESS"
		
		if info ~= nil then
			response["info"] = infoMessage
		end
		
		return response
	end

	local request = setSVDRequest(paramsSend)
	local response = setSVDResponse(paramsSend)
	
	if infoMessage ~= nil then
		response["info"] = infoMessage
	end
	
	--mobile side: sending SubscribeVehicleData request
	local cid = self.mobileSession:SendRPC("SubscribeVehicleData",request)
	
	--hmi side: expect SubscribeVehicleData request
	EXPECT_HMICALL("VehicleInfo.SubscribeVehicleData",request)
	:Do(function(_,data)
		--hmi side: sending VehicleInfo.SubscribeVehicleData response
		self.hmiConnection:SendResponse(data.id, data.method, "SUCCESS", response)	
	end)
	
	
	local expectedResult = createSuccessExpectedResult(response, infoMessage)
	
	--mobile side: expect SubscribeVehicleData response
	EXPECT_RESPONSE(cid, expectedResult)
	
	--mobile side: expect OnHashChange notification
	EXPECT_NOTIFICATION("OnHashChange")
end


---------------------------------------------------------------------------------------------
-------------------------------------------Preconditions-------------------------------------
---------------------------------------------------------------------------------------------



	--Print new line to separate new test cases group
	commonFunctions:newTestCasesGroup("Preconditions")
	
	--1. Activate application
	commonSteps:ActivationApp()

	local PermissionLines_SubscribeVehicleData = 
[[					"SubscribeVehicleData": {
						"hmi_levels": [
						  "BACKGROUND",
						  "FULL",
						  "LIMITED"
						],
						"parameters": [
							"gps", 
							"speed", 
							"rpm", 
							"fuelLevel", 
							"fuelLevel_State", 
							"instantFuelConsumption", 
							"externalTemperature", 
							"prndl", 
							"tirePressure", 
							"odometer", 
							"beltStatus", 
							"bodyInformation", 
							"deviceStatus", 
							"driverBraking", 
							"wiperStatus", 
							"headLampStatus", 
							"engineTorque", 
							"accPedalPosition", 
							"steeringWheelAngle", 
							"eCallInfo", 
							"airbagStatus", 
							"emergencyEvent", 
							"clusterModeStatus", 
							"myKey"
					   ]
					  }]]


	local PermissionLines_OnVehicleData = 
[[					"OnVehicleData": {
						"hmi_levels": [
						  "BACKGROUND",
						  "FULL",
						  "LIMITED"
						],
						"parameters": [
							"gps", 
							"speed",  
							"rpm", 
							"fuelLevel", 
							"fuelLevel_State", 
							"instantFuelConsumption", 
							"externalTemperature", 
							"prndl", 
							"tirePressure", 
							"odometer", 
							"beltStatus", 
							"bodyInformation", 
							"deviceStatus", 
							"driverBraking", 
							"wiperStatus", 
							"headLampStatus", 
							"engineTorque", 
							"accPedalPosition", 
							"steeringWheelAngle", 
							"eCallInfo", 
							"airbagStatus", 
							"emergencyEvent", 
							"clusterModeStatus", 
							"myKey"
					   ]
					  }]]
		

	-- local PermissionLinesForBase4 = PermissionLines_SubscribeVehicleData .. ", \n" .. PermissionLines_OnVehicleData ..", \n"
	-- local PermissionLinesForGroup1 = nil
	-- local PermissionLinesForApplication = nil
	-- local PTName = testCasesForPolicyTable:createPolicyTableFile(PermissionLinesForBase4, PermissionLinesForGroup1, PermissionLinesForApplication)	
	-- testCasesForPolicyTable:updatePolicy(PTName)		
	
	

	function Test:SubscribeVehicleData_Positive() 				
		self:subscribeVehicleDataSuccess(allVehicleData)				
	end
	
	



	Test["OnVehicleData_prndl_PARK"] = function(self)
	
		
		self.hmiConnection:SendNotification("VehicleInfo.OnVehicleData", {prndl = "PARK"})	  		

		--mobile side: expected SubscribeVehicleData response
		EXPECT_NOTIFICATION("OnVehicleData",  {prndl = "PARK"})	
		
	end
	
	Test["OnVehicleData_prndl_FAULT"] = function(self)
	
		
		self.hmiConnection:SendNotification("VehicleInfo.OnVehicleData", {prndl = "FAULT"})	  		

		--mobile side: expected SubscribeVehicleData response
		EXPECT_NOTIFICATION("OnVehicleData",  {prndl = "FAULT"})	
		
	end
		